/* eslint-disable @typescript-eslint/no-namespace */
// tslint:disable-next-line: no-namespace
export var Draw;
(function (Draw) {
    class DrawAny {
        constructor(entity, options) {
            if (entity !== undefined) {
                this.entity = entity;
            }
            if (options !== undefined) {
                this.options = options;
            }
        }
    }
    Draw.DrawAny = DrawAny;
    class DrawOcctShapeOptions {
        /**
         * Provide options without default values
         */
        constructor(faceOpacity, edgeOpacity, edgeColour, faceMaterial, faceColour, edgeWidth, drawEdges, drawFaces, drawVertices, vertexColour, vertexSize, precision, drawEdgeIndexes, edgeIndexHeight, edgeIndexColour, drawFaceIndexes, faceIndexHeight, faceIndexColour) {
            /**
             * Face opacity value between 0 and 1
             * @default 1
             * @minimum 0
             * @maximum 1
             * @step 0.1
             */
            this.faceOpacity = 1;
            /**
             * Edge opacity value between 0 and 1
             * @default 1
             * @minimum 0
             * @maximum 1
             * @step 0.1
             */
            this.edgeOpacity = 1;
            /**
             * Hex colour string for the edges
             * @default #ffffff
             */
            this.edgeColour = "#ffffff";
            /**
             * Hex colour string for face colour
             * @default #ff0000
             */
            this.faceColour = "#ff0000";
            /**
             * Color of the vertices that will be drawn
             * @default #ff00ff
             */
            this.vertexColour = "#ffaaff";
            /**
             * Edge width
             * @default 2
             * @minimum 0
             * @maximum Infinity
             * @step 0.1
             */
            this.edgeWidth = 2;
            /**
             * The size of a vertices that will be drawn
             * @default 0.03
             * @minimum 0
             * @maximum Infinity
             * @step 0.01
             */
            this.vertexSize = 0.03;
            /**
             * You can turn off drawing of edges via this property
             * @default true
             */
            this.drawEdges = true;
            /**
             * You can turn off drawing of faces via this property
             * @default true
             */
            this.drawFaces = true;
            /**
             * You can turn off drawing of vertexes via this property
             * @default false
             */
            this.drawVertices = false;
            /**
             * Precision of the mesh that will be generated for the shape, lower number will mean more triangles
             * @default 0.01
             * @minimum 0
             * @maximum Infinity
             * @step 0.01
             */
            this.precision = 0.01;
            /**
             * Draw index of edges in space
             * @default false
             */
            this.drawEdgeIndexes = false;
            /**
             * Indicates the edge index height if they are drawn
             * @default 0.06
             * @minimum 0
             * @maximum Infinity
             * @step 0.01
             */
            this.edgeIndexHeight = 0.06;
            /**
             * Edge index colour if the edges are drawn
             * @default #ff00ff
             */
            this.edgeIndexColour = "#ff00ff";
            /**
             * Draw indexes of faces in space
             * @default false
             */
            this.drawFaceIndexes = false;
            /**
             * Indicates the edge index height if they are drawn
             * @default 0.06
             * @minimum 0
             * @maximum Infinity
             * @step 0.01
             */
            this.faceIndexHeight = 0.06;
            /**
             * Edge index colour if the edges are drawn
             * @default #0000ff
             */
            this.faceIndexColour = "#0000ff";
            if (faceOpacity !== undefined) {
                this.faceOpacity = faceOpacity;
            }
            if (edgeOpacity !== undefined) {
                this.edgeOpacity = edgeOpacity;
            }
            if (edgeColour !== undefined) {
                this.edgeColour = edgeColour;
            }
            if (faceMaterial !== undefined) {
                this.faceMaterial = faceMaterial;
            }
            if (faceColour !== undefined) {
                this.faceColour = faceColour;
            }
            if (vertexColour !== undefined) {
                this.vertexColour = vertexColour;
            }
            if (vertexSize !== undefined) {
                this.vertexSize = vertexSize;
            }
            if (edgeWidth !== undefined) {
                this.edgeWidth = edgeWidth;
            }
            if (drawEdges !== undefined) {
                this.drawEdges = drawEdges;
            }
            if (drawFaces !== undefined) {
                this.drawFaces = drawFaces;
            }
            if (drawVertices !== undefined) {
                this.drawVertices = drawVertices;
            }
            if (precision !== undefined) {
                this.precision = precision;
            }
            if (drawEdgeIndexes !== undefined) {
                this.drawEdgeIndexes = drawEdgeIndexes;
            }
            if (edgeIndexHeight !== undefined) {
                this.edgeIndexHeight = edgeIndexHeight;
            }
            if (edgeIndexColour !== undefined) {
                this.edgeIndexColour = edgeIndexColour;
            }
            if (drawFaceIndexes !== undefined) {
                this.drawFaceIndexes = drawFaceIndexes;
            }
            if (faceIndexHeight !== undefined) {
                this.faceIndexHeight = faceIndexHeight;
            }
            if (faceIndexColour !== undefined) {
                this.faceIndexColour = faceIndexColour;
            }
        }
    }
    Draw.DrawOcctShapeOptions = DrawOcctShapeOptions;
    /**
     * Draw options for basic geometry types like points, lines, polylines, surfaces and jscad meshes
     */
    class DrawBasicGeometryOptions {
        constructor(colours, size, opacity, updatable, hidden) {
            /**
             * Basic geometry colours to use for lines, points, polylines, surfaces, jscad meshes.
             * @default #ff0000
             */
            this.colours = "#ff0000";
            /**
             * Size affect how big the drawn points are and how wide lines are.
             * @default 0.1
             * @minimum 0
             * @maximum Infinity
             * @step 0.1
             */
            this.size = 0.1;
            /**
             * Opacity of the point 0 to 1
             * @default 1
             * @minimum 0
             * @maximum 1
             * @step 0.1
             */
            this.opacity = 1;
            /**
             * If geometry needs to be updated later
             * @default false
             */
            this.updatable = false;
            /**
             * Hidden
             * @default false
             */
            this.hidden = false;
            if (colours !== undefined) {
                this.colours = colours;
            }
            if (size !== undefined) {
                this.size = size;
            }
            if (opacity !== undefined) {
                this.opacity = opacity;
            }
            if (updatable !== undefined) {
                this.updatable = updatable;
            }
            if (hidden !== undefined) {
                this.hidden = hidden;
            }
        }
    }
    Draw.DrawBasicGeometryOptions = DrawBasicGeometryOptions;
    let drawingTypes;
    (function (drawingTypes) {
        drawingTypes[drawingTypes["point"] = 0] = "point";
        drawingTypes[drawingTypes["points"] = 1] = "points";
        drawingTypes[drawingTypes["line"] = 2] = "line";
        drawingTypes[drawingTypes["lines"] = 3] = "lines";
        drawingTypes[drawingTypes["node"] = 4] = "node";
        drawingTypes[drawingTypes["nodes"] = 5] = "nodes";
        drawingTypes[drawingTypes["polyline"] = 6] = "polyline";
        drawingTypes[drawingTypes["polylines"] = 7] = "polylines";
        drawingTypes[drawingTypes["verbCurve"] = 8] = "verbCurve";
        drawingTypes[drawingTypes["verbCurves"] = 9] = "verbCurves";
        drawingTypes[drawingTypes["verbSurface"] = 10] = "verbSurface";
        drawingTypes[drawingTypes["verbSurfaces"] = 11] = "verbSurfaces";
        drawingTypes[drawingTypes["jscadMesh"] = 12] = "jscadMesh";
        drawingTypes[drawingTypes["jscadMeshes"] = 13] = "jscadMeshes";
        drawingTypes[drawingTypes["occt"] = 14] = "occt";
        drawingTypes[drawingTypes["occtShapes"] = 15] = "occtShapes";
        drawingTypes[drawingTypes["tag"] = 16] = "tag";
        drawingTypes[drawingTypes["tags"] = 17] = "tags";
    })(drawingTypes = Draw.drawingTypes || (Draw.drawingTypes = {}));
})(Draw || (Draw = {}));
